from ftw.builder import Builder
from ftw.builder import create
from ftw.contacts.contents.contact import IContactSchema
from ftw.contacts.interfaces import IContact
from ftw.contacts.testing import FTW_CONTACTS_FUNCTIONAL_TESTING
from ftw.contacts.testing import FTW_CONTACTS_GEO_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from unittest2 import TestCase
from z3c.relationfield.interfaces import IHasIncomingRelations
from zope.component import createObject
from zope.component import queryUtility


class TestDefaultView(TestCase):
    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))

    @browsing
    def test_call_view_does_not_fail(self, browser):
        browser.login().visit(
            self.contactfolder, view="++add++ftw.contacts.Contact")

        browser.fill({
            'Organization': '',
            'Firstname': 'Chuck',
            'Lastname': 'N\xc3\xb6rris'}).submit()

        self.assertEqual(1, len(browser.css('#contact-view')))

    @browsing
    def test_do_not_show_map_widget(self, browser):
        browser.login().visit(
            self.contactfolder, view="++add++ftw.contacts.Contact")

        browser.fill({
            'Firstname': 'Chuck',
            'Lastname': 'N\xc3\xb6rris'}).submit()

        self.assertEqual(0, len(browser.css('.mapWidget')))

    @browsing
    def test_show_memberships_of_contact(self, browser):
        contact = create(Builder('contact')
                         .with_minimal_info(u'Ch\xf6ck', u'4orris')
                         .within(self.contactfolder))

        create(Builder('member block')
               .within(self.contactfolder)
               .contact(contact)
               .titled(u"A MemberBlock"))

        create(Builder('member block')
               .within(self.contactfolder)
               .contact(contact)
               .titled(u"A MemberBlock"))

        browser.login().visit(contact)

        self.assertEqual(2, len(browser.css('.memberships li')))

        # Now hide the memberships
        browser.visit(contact, view='edit')
        browser.fill({u'Hide memberships': True}).submit()
        self.assertEqual([], browser.css('.memberships'))

    @browsing
    def test_memberships_sorted_alphabetically(self, browser):
        """
        This test makes sure that the back references (AKA member ships)
        of a contact are sorted alphabetically by the title of their
        container.
        """
        contact = create(Builder('contact')
                         .with_minimal_info(u'John', u'Doe')
                         .within(self.contactfolder))

        zzz = create(Builder('sl content page').titled(u'ZZZ'))
        create(Builder('member block')
               .within(zzz)
               .contact(contact))

        aaa = create(Builder('sl content page').titled(u'AAA'))
        create(Builder('member block')
               .within(aaa)
               .contact(contact))

        ttt = create(Builder('sl content page').titled(u'TTT'))
        create(Builder('member block')
               .within(ttt)
               .contact(contact))

        browser.login().visit(contact)
        self.assertEqual(
            [
                'AAA',
                'TTT',
                'ZZZ',
            ],
            browser.css('.memberships li').text
        )

    @browsing
    def test_membership_links_point_to_container(self, browser):
        """
        This test makes sure that the membership links on the contact
        point to the container of the memberblock and not the
        memberblock itself.
        """
        contact = create(Builder('contact')
                         .with_minimal_info(u'Ch\xf6ck', u'4orris')
                         .within(self.contactfolder))

        page = create(Builder('sl content page').titled(u'Team'))

        create(Builder('member block')
               .within(page)
               .contact(contact)
               .titled(u"A MemberBlock")
               .having(function=u'Epic splitter'))

        browser.login().visit(contact)
        self.assertEqual(
            'http://nohost/plone/team#a-memberblock',
            browser.css('.memberships li a').first.attrib['href']
        )


class TestIdGeneration(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    @browsing
    def test_generate_id_from_last_and_firstname(self, browser):
        contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))
        browser.login().visit(
            contactfolder, view="++add++ftw.contacts.Contact")

        browser.fill({
            'Organization': '',
            'Firstname': 'Chuck',
            'Lastname': 'N\xc3\xb6rris'}).submit()

        self.assertEqual(
            1, len(contactfolder.listFolderContents()),
            "Something went wrong while creating the contact")

        self.assertEqual(
            "norris-chuck", contactfolder.listFolderContents()[0].id)

    @browsing
    def test_generate_id_from_organization(self, browser):
        contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))
        browser.login().visit(
            contactfolder, view="++add++ftw.contacts.Contact")

        browser.fill({
            'Organization': 'CIA Headquarter',
            'Firstname': '',
            'Lastname': ''}).submit()

        self.assertEqual(
            1, len(contactfolder.listFolderContents()),
            "Something went wrong while creating the contact")

        self.assertEqual(
            "cia-headquarter", contactfolder.listFolderContents()[0].id)


class TestTitleGeneration(TestCase):
    """
    """
    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    @browsing
    def test_generate_title_from_last_and_firstname(self, browser):
        contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))
        browser.login().visit(
            contactfolder, view="++add++ftw.contacts.Contact")

        browser.fill({
            'Organization': '',
            'Firstname': 'Chuck',
            'Lastname': 'N\xc3\xb6rris'}).submit()

        self.assertEqual(
            1, len(contactfolder.listFolderContents()),
            "Something went wrong while creating the contact")

        self.assertEqual(
            u'N\xf6rris Chuck', contactfolder.listFolderContents()[0].title)

    @browsing
    def test_generate_title_from_organization(self, browser):
        contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))
        browser.login().visit(
            contactfolder, view="++add++ftw.contacts.Contact")

        browser.fill({
            'Organization': 'CIA Headquarter',
            'Firstname': '',
            'Lastname': ''}).submit()

        self.assertEqual(
            1, len(contactfolder.listFolderContents()),
            "Something went wrong while creating the contact")

        self.assertEqual(
            u"CIA Headquarter", contactfolder.listFolderContents()[0].title)

    @browsing
    def test_setting_title_does_nothing(self, browser):
        contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))
        browser.login().visit(
            contactfolder, view="++add++ftw.contacts.Contact")

        browser.fill({
            'Organization': 'CIA Headquarter',
            'Firstname': '',
            'Lastname': ''}).submit()

        contact = contactfolder.listFolderContents()[0]
        contact.setTitle("Chucks Headquarter")

        self.assertEqual(
            u"CIA Headquarter", contactfolder.listFolderContents()[0].title)


class TestContactValidateOrganizationOrFullname(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    @browsing
    def test_error_if_no_first_last_and_org_name(self, browser):
        contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))
        browser.login().visit(
            contactfolder, view="++add++ftw.contacts.Contact")

        browser.fill({
            'Organization': '',
            'Firstname': '',
            'Lastname': ''}).submit()

        self.assertEqual(1, len(browser.css('.portalMessage.error')))
        self.assertTrue(browser.url.endswith('++add++ftw.contacts.Contact'))

    @browsing
    def test_error_if_only_firstname_exists(self, browser):
        contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))
        browser.login().visit(
            contactfolder, view="++add++ftw.contacts.Contact")

        browser.fill({'Firstname': "Chuck"}).submit()

        self.assertEqual(1, len(browser.css('.portalMessage.error')))
        self.assertTrue(browser.url.endswith('++add++ftw.contacts.Contact'))

    @browsing
    def test_error_if_only_lastname_exists(self, browser):
        contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))
        browser.login().visit(
            contactfolder, view="++add++ftw.contacts.Contact")

        browser.fill({'Lastname': "Norris"}).submit()

        self.assertEqual(1, len(browser.css('.portalMessage.error')))
        self.assertTrue(browser.url.endswith('++add++ftw.contacts.Contact'))

    @browsing
    def test_no_error_if_org_name_exists(self, browser):
        contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))
        browser.login().visit(
            contactfolder, view="++add++ftw.contacts.Contact")

        browser.fill({'Organization': "CIA"}).submit()

        self.assertEqual(0, len(browser.css('.portalMessage.error')))
        self.assertEqual(
            1, len(contactfolder.listFolderContents()),
            "Something went wrong while creating the contact")

    @browsing
    def test_no_error_if_first_and_last_name_exists(self, browser):
        contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))
        browser.login().visit(
            contactfolder, view="++add++ftw.contacts.Contact")

        browser.fill({
            'Firstname': "Chuck",
            'Lastname': "N\xc3\xb6rris"}).submit()

        self.assertEqual(0, len(browser.css('.portalMessage.error')))
        self.assertEqual(
            1, len(contactfolder.listFolderContents()),
            "Something went wrong while creating the contact")


class TestContactInstallation(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_adding(self):
        contact = create(Builder('contact'))
        self.assertTrue(IContact.providedBy(contact))

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name='ftw.contacts.Contact')
        self.assertNotEquals(None, fti)

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name='ftw.contacts.Contact')
        schema = fti.lookupSchema()
        self.assertEqual(IContactSchema, schema)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name='ftw.contacts.Contact')
        factory = fti.factory
        new_object = createObject(factory)
        self.assertTrue(IContactSchema.providedBy(new_object))

    def test_relation(self):
        contact = create(Builder('contact'))
        self.assertTrue(IHasIncomingRelations.providedBy(contact))


class TestDefaultGeoView(TestCase):
    layer = FTW_CONTACTS_GEO_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))

    @browsing
    def test_show_map_widget(self, browser):
        browser.login().visit(
            self.contactfolder, view="++add++ftw.contacts.Contact")

        browser.fill({
            'Firstname': 'Chuck',
            'Lastname': 'N\xc3\xb6rris'}).submit()

        self.assertEqual(1, len(browser.css('.mapWidget')))

        # Now hide map
        browser.find('Edit').click()
        browser.fill({u'Hide map': True}).submit()
        self.assertEqual([], browser.css('.mapWidget'))



from ftw.contacts.simplelayout.member import MemberView
from ftw.contacts.utils import get_contact_title
from ftw.simplelayout.browser.blocks.base import BaseBlock
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class MemberBlockView(BaseBlock, MemberView):

    template = ViewPageTemplateFile('templates/memberblock.pt')

    @property
    def image_caption(self):
        mtool = getToolByName(self, "portal_membership")

        if mtool.checkPermission('View', self.context.contact.to_object):
            return self.contact_title
        return ''

    @property
    def contact_title(self):
        return get_contact_title(
            self.context.contact.to_object, display='natural')



from ftw.builder import builder_registry
from ftw.builder.dexterity import DexterityBuilder
from plone.app.textfield.value import RichTextValue
from plone.namedfile import NamedImage
from z3c.relationfield.relation import create_relation
from ftw.simplelayout.tests import builders


class ContactFolderBuilder(DexterityBuilder):
    portal_type = 'ftw.contacts.ContactFolder'

builder_registry.register('contact folder', ContactFolderBuilder)


class ContactBuilder(DexterityBuilder):
    portal_type = 'ftw.contacts.Contact'

    def with_image(self):
        image_data = (
            'GIF89a\x01\x00\x01\x00\x80\x00\x00\x00\x00\x00\x00\x00'
            '\x00!\xf9\x04\x04\x00\x00\x00\x00,\x00\x00\x00\x00\x01\x00'
            '\x01\x00\x00\x02\x02D\x01\x00;'
        )
        self.arguments["image"] = NamedImage(image_data, filename=u'image.gif')
        return self

    def with_minimal_info(self, firstname, lastname):
        """Contact with ontly the required fields filled
        """
        self.arguments.update(
            lastname=lastname,
            firstname=firstname
            )

        return self

    def with_maximal_info(self, firstname, lastname):
        """Contact with all possible information
        """
        self.arguments.update(
            organization=u'Fant\xe0storg',
            gender='f',
            lastname=lastname,
            firstname=firstname,
            address=u'Chrache zw\xf6i',
            postal_code='1337',
            city=u'G\xf4tham',
            country='Schweiz',
            email='fmeier@stirnimaa.ch',
            phone_office='+41 33 456 78 01',
            phone_mobile='+70 98 765 43 21',
            fax='+41 33 456 78 02',
            www='http://www.cheib.ch',
            academic_title='Master of the Universe',
            function=u'Imk\xe9r',
            department=u'Cust\xf5mer Services',
            salutation='Sir',
            text=RichTextValue(u'He is \xb1 awesome!'),
            phone_private='+41 70 123 32 12',
            address_private='Chriesleweg 5',
            postal_code_private='9999',
            city_private='Dubai')

        self.with_image()

        return self

builder_registry.register('contact', ContactBuilder)


class MemberBlockBuilder(DexterityBuilder):
    portal_type = 'ftw.contacts.MemberBlock'

    def contact(self, contact):
        self.arguments.update(
            contact=create_relation(contact.getPhysicalPath()))

        return self

builder_registry.register('member block', MemberBlockBuilder)



from ftw.builder import Builder
from ftw.builder import create
from ftw.contacts.testing import FTW_CONTACTS_FUNCTIONAL_TESTING
from ftw.contacts.utils import get_organization
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest2 import TestCase


class TestGetOrganization(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))

    def test_return_empty_if_no_portal_types_are_defined(self):

        self.assertEqual("", get_organization(self.contactfolder))

    def test_return_empty_if_portal_type_not_found(self):
        self.assertEqual(
            "", get_organization(self.contactfolder, ['ftw.contacts.Contact']))

    def test_return_title_of_found_portal_type(self):
        contact = create(Builder('contact')
                         .with_minimal_info(u'Ch\xf6ck', u'4orris')
                         .within(self.contactfolder))

        self.assertEqual(
            "Contact folder",
            get_organization(contact, ['ftw.contacts.ContactFolder']))



from ftw.builder import Builder
from ftw.builder import create
from ftw.contacts.interfaces import IMemberAccessor
from ftw.contacts.testing import FTW_CONTACTS_FUNCTIONAL_TESTING
from Products.CMFCore.utils import getToolByName
from unittest2 import TestCase


class TestMemberAccessor(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.catalog = getToolByName(self.portal, 'portal_catalog')
        self.contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))

    def test_get_attr_on_member_if_not_aquire_address(self):
        contact = create(Builder('contact')
                         .with_minimal_info(u'Chuck', u'4\xf6rris')
                         .having(address='Bundesplatz 1')
                         .within(self.contactfolder))

        member_block = create(Builder('member block')
                              .within(self.contactfolder)
                              .contact(contact)
                              .having(address='Frankenstrasse 53'))

        self.assertEqual(u'Frankenstrasse 53', IMemberAccessor(member_block).address)

    def test_get_attr_on_contact_if_aquire_address(self):
        contact = create(Builder('contact')
                         .with_minimal_info(u'Chuck', u'4\xf6rris')
                         .having(address='Bundesplatz 1')
                         .within(self.contactfolder))

        member_block = create(Builder('member block')
                              .within(self.contactfolder)
                              .contact(contact)
                              .having(address='Frankenstrasse 53',
                                      acquire_address=True))

        self.assertEqual(u'Bundesplatz 1', IMemberAccessor(member_block).address)

    def test_get_attr_on_member_if_aquire_address_but_attr_not_exist(self):
        contact = create(Builder('contact')
                         .with_minimal_info(u'Chuck', u'4\xf6rris')
                         .within(self.contactfolder))

        member_block = create(Builder('member block')
                              .within(self.contactfolder)
                              .contact(contact)
                              .having(
                                  acquire_address=True,
                                  show_title=True))

        self.assertEqual(False, hasattr(contact, 'show_title'))
        self.assertEqual(True, IMemberAccessor(member_block).show_title)

    def test_get_not_existing_attr_returns_none(self):
        contact = create(Builder('contact')
                         .with_minimal_info(u'Chuck', u'4\xf6rris')
                         .within(self.contactfolder))

        member_block = create(Builder('member block')
                              .within(self.contactfolder)
                              .contact(contact))

        self.assertEqual(None, IMemberAccessor(member_block).not_existing_attr)



"""Migration from egov.contactdirectory to ftw.contacts.

The migration is not registered / executed automatically.
It must be executed manually from an upgrade step in an upgrade step of the
integration project.
"""

from ftw.upgrade import UpgradeStep
from ftw.upgrade.migration import InplaceMigrator


# Some sources will probably be removed after running the migration.
# But if the imports are kept and the sources are removed, the imports
# may raise ImportErrors. We only want to have those ImportErrors when
# we actually need to run the migration.
try:

    from ftw.upgrade.migration import DUBLIN_CORE_IGNORES

except ImportError, IMPORT_ERROR:
    pass
else:
    IMPORT_ERROR = None


class ExampleUpgradeStep(UpgradeStep):
    """This is just an example how an upgrade could be done in an upgrade
    step in the integration package.
    """

    def __call__(self):
        page_migrator = ContactMigrator()
        map(page_migrator.migrate_object,
            self.objects({'query': 'Contact'}), 'Migrate contacts')


class ContactMigrator(InplaceMigrator):

    def __init__(self, ignore_fields=(), additional_steps=(), options=0):
        if IMPORT_ERROR:
            raise IMPORT_ERROR

        super(ContactMigrator, self).__init__(
            new_portal_type='ftw.contacts.Contact',
            ignore_fields=(
                DUBLIN_CORE_IGNORES
                + ignore_fields + (
                    'description',  # invisible
                    'title',  # invisible
                    'effectiveDate',  # publication behavior not enabled
                    'expirationDate',  # publication behavior not enabled
                    'excludeFromNav',  # exclude from nav not enabled
                    'showPlacemark',  # field removed
                    'show_memberships',  # field removed
                    'subject',  # unused and field removed
                )),
            field_mapping={
                'tel_private': 'phone_private',
                'zip': 'postal_code',
                'zip_private': 'postal_code_private'},
            additional_steps=additional_steps,
            options=options,
        )

    def query(self, path=None):
        query = {'portal_type': 'Contact'}
        if path:
            query['path'] = path
        return query


class MemberMigrator(InplaceMigrator):
    """Requires the simplelayout profile to be installed.
    """

    def __init__(self, ignore_fields=(), additional_steps=(), options=0):
        if IMPORT_ERROR:
            raise IMPORT_ERROR

        super(MemberMigrator, self).__init__(
            new_portal_type='ftw.contacts.MemberBlock',
            ignore_fields=(
                DUBLIN_CORE_IGNORES
                + ignore_fields + (
                    'effectiveDate',  # unsuitable for blocks
                    'expirationDate',  # unsuitable for blocks
                    'excludeFromNav',  # unsuitable for blocks
                    'description',  # unnecessary
                    'subject',  # unnecessary
                )),
            field_mapping={
                'acquireAddress': 'acquire_address',
                'showTitle': 'show_title',
                'zip': 'postal_code'},
            additional_steps=additional_steps,
            options=options,
        )

    def query(self, path=None):
        query = {'portal_type': 'Member'}
        if path:
            query['path'] = path
        return query



from ftw.builder import Builder
from ftw.builder import create
from ftw.contacts.browser.vcard import DownloadVCardView
from ftw.contacts.testing import FTW_CONTACTS_FUNCTIONAL_TESTING
from ftw.contacts.utils import generate_vcard
from unittest2 import TestCase

import os


def asset(filename):
    here = os.path.dirname(__file__)
    path = os.path.join(here, 'assets', filename)
    with open(path, 'r') as file_:
        return file_.read()


class TextVCardExport(TestCase):
    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    def test_detailed_contact_works(self):
        contact = create(Builder('contact').with_maximal_info(
            firstname=u'Fr\xedtz',
            lastname=u'M\xe9ier'
            ))

        self.assertMultiLineEqual(
            asset('fritz-meier.vcf'), generate_vcard(contact).getvalue())

    def test_minimal_contact_works(self):
        contact = create(Builder('contact').with_minimal_info(
            firstname='Minho',
            lastname='Blau'))

        self.assertMultiLineEqual(
            asset('blau-minho.vcf'), generate_vcard(contact).getvalue())

    def test_call_vcard_download_view(self):
        contact = create(Builder('contact').with_minimal_info(
            firstname='Minho',
            lastname='Blau'))

        vcard_view = DownloadVCardView(contact, self.request)

        self.assertMultiLineEqual(asset('blau-minho.vcf'), vcard_view())



class ContactLocationAdapter(object):
    """Adapter that is able to represent the location of Contact in
    a geocodable string form.
    """

    def __init__(self, context):
        self.context = context

    def getLocationString(self):
        """Build a geocodable location string from the Contact address
        related fields.
        """
        street = u' '.join(self.context.address.strip().split())
        # Remove Postfach from street, otherwise Google geocoder API will
        # return wrong results
        street = street.replace(u'Postfach', u'').replace(u'\r', u'').strip()
        zip_code = self.context.postal_code.strip()
        city = self.context.city.strip()
        country = self.context.country.strip()

        # We need at least something other than country to be defined,
        # otherwise we can't do a meaningful geocode lookup
        if not (street or zip_code or city):
            return None

        # Concatenate only the fields with a value into the location string
        return u', '.join(filter(None, [street, zip_code, city, country]))



from zope.i18nmessageid import MessageFactory
from zope.interface import Interface
from zope import schema


_ = MessageFactory('plone')


class IMemberAccessor(Interface):
    """Generic member accessor adapter implementation for Dexterity content
       objects.
    """


class IContactFolder(Interface):
    """
    """


class IContact(Interface):
    """
    """


class IMemberBlock(Interface):
    """
    """


class ILDAPAttributeMapper(Interface):
    """A utility that provides the mapping between LDAP attributes and
       Dexterity fields.
    """
    def mapping():
        """Returns a mapping of LDAP attibute names -> DX field names.
        """

    def id():
        """Returns the name of the LDAP attribute name used as contact id.
        """


class ILDAPCustomUpdater(Interface):
    """An adapter for updating a contact object with custom data.
       Adapts a contact object and the related ldap record.
    """
    def update():
        """Updates the adapted contact object. Returns true if the object was
           modified.
        """


class ILDAPSearch(Interface):
    """Utility for searching in LDAP.
    """


class IContactsSettings(Interface):
    """Registry entries for ftw.contacts"""

    ldap_plugin_id = schema.TextLine(
        title=_(u'LDAP Plugin ID'),
        description=_(u'ID of the LDAP PAS plugin to use '
                      u'for contact synchronisation'),
        default=None,
        required=False)

    contacts_path = schema.TextLine(
        title=_(u'Contacts Path'),
        description=_(u'Path to the folder containing contacts'),
        default=None,
        required=False)



from ftw.builder import Builder
from ftw.builder import create
from ftw.contacts.interfaces import IMemberBlock
from ftw.contacts.testing import FTW_CONTACTS_FUNCTIONAL_TESTING
from ftw.contacts.utils import get_backreferences
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest2 import TestCase


class TestBackReferences(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Site Administrator'])

        self.contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))

    def test_references_in_a_list(self):
        contact = create(Builder('contact')
                         .with_minimal_info(u'Ch\xf6ck', u'4orris')
                         .within(self.contactfolder))

        member_block = create(Builder('member block')
                              .within(self.contactfolder)
                              .contact(contact)
                              .having(
                                  firstname=u"J\xf6mes"))

        self.assertEqual(
            [member_block], get_backreferences(contact, IMemberBlock))

    def test_empty_list_if_no_references_found(self):
        contact = create(Builder('contact')
                         .with_minimal_info(u'Ch\xf6ck', u'4orris')
                         .within(self.contactfolder))

        self.assertEqual([], get_backreferences(contact, IMemberBlock))

    def test_do_not_append_objs_with_no_permission(self):
        contact = create(Builder('contact')
                         .with_minimal_info(u'Ch\xf6ck', u'4orris')
                         .within(self.contactfolder))

        member_block = create(Builder('member block')
                              .within(self.contactfolder)
                              .contact(contact)
                              .having(
                                  firstname=u"J\xf6mes"))

        self.assertEqual(
            [member_block], get_backreferences(contact, IMemberBlock))

        member_block.manage_permission('View', roles=[])
        logout()

        self.assertEqual([], get_backreferences(contact, IMemberBlock))



from ftw.builder import Builder
from ftw.builder import create
from ftw.contacts.testing import FTW_CONTACTS_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from StringIO import StringIO
from unittest2 import TestCase
from zipfile import ZipFile

import os


def asset(filename):
    here = os.path.dirname(__file__)
    path = os.path.join(here, 'assets', filename)
    with open(path, 'r') as file_:
        return file_.read()


class TextVCardExport(TestCase):
    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    @browsing
    def test_zip_export(self, browser):
        contact = create(
            Builder('contact').with_maximal_info(u'Fr\xedtz', u'M\xe9ier'))

        browser.login().visit(contact, view='zip_export')
        self.assertEquals('application/zip', browser.headers['Content-Type'])

        zipfile = ZipFile(StringIO(browser.contents))
        self.assertEquals(['meier-fritz.vcf'], zipfile.namelist())
        self.assertMultiLineEqual(asset('fritz-meier.vcf'),
                                  zipfile.read('meier-fritz.vcf'))



class DefaultLDAPAttributeMapper(object):

    def mapping(self):
        return {
            'uid': 'uid',
            'o': 'organization',
            'sn': 'lastname',
            'givenName': 'firstname',
            'street': 'address',
            'postalCode': 'postal_code',
            'l': 'city',
            'c': 'country',
            'mail': 'email',
            'telephoneNumber': 'phone_office',
            'mobile': 'phone_mobile',
            'facsimileTelephoneNumber': 'fax',
            'department': 'department',
        }

    def id(self):
        return 'uid'



from ftw.contacts.testing import FTW_CONTACTS_FUNCTIONAL_TESTING
from ftw.contacts.utils import encode
from unittest2 import TestCase


class TestContactTitle(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def test_encode_unicode_to_utf8(self):
        self.assertEqual("\xc3\xbcber", encode(u"\xfcber"))

    def test_return_utf8_strings(self):
        self.assertEqual("\xc3\xbcber", encode("\xc3\xbcber"))

    def test_return_empty_string_if_text_is_none(self):
        self.assertEqual("", encode(None))



from ftw.upgrade import UpgradeStep


class AddMemberRegistryForSourcePath(UpgradeStep):
    """Add member registry for source path.
    """

    def __call__(self):
        self.install_upgrade_profile()



from collective import dexteritytextindexer
from ftw.contacts import _
from ftw.contacts.interfaces import IMemberBlock
from ftw.contacts.simplelayout.interfaces import IMemberRegistry
from ftw.contacts.utils import get_organization
from ftw.referencewidget.widget import ReferenceBrowserWidget
from plone.dexterity.content import Item
from plone.directives.form import widget
from plone.registry.interfaces import IRegistry
from plone.supermodel import model
from z3c.relationfield.schema import Relation
from z3c.schema.email import RFC822MailAddress
from zope import schema
from zope.component import getUtility
from zope.interface import implements


def get_address_root(widget):
    registry = getUtility(IRegistry).forInterface(IMemberRegistry)
    path = registry.member_nav_tree_query_path or ''

    if not path:
        path = '/'.join(widget.context.getPhysicalPath())
    return path


class IMemberBlockSchema(model.Schema):
    """A member type schema interface
    """
    dexteritytextindexer.searchable('title')
    title = schema.TextLine(
        title=_(u'label_title', default=u'Title'),
        missing_value=u"",
        default=u"",
        required=False)

    show_title = schema.Bool(
        title=_(u'label_show_title', default=u'Show title'),
        required=False)

    widget('contact', ReferenceBrowserWidget, start=get_address_root,
           override=True, selectable=['ftw.contacts.Contact'])
    contact = Relation(
        title=_(u'label_contact_reference', default=u'Contact reference'),
        required=True)

    dexteritytextindexer.searchable('function')
    function = schema.TextLine(
        title=_(u'label_function', default=u'Function'),
        missing_value=u"",
        default=u"",
        required=False)

    show_address = schema.Bool(
        title=_(u'label_show_address', default=u'Show address'),
        required=False)

    show_image = schema.Bool(
        title=_(u'label_show_image', default=u'Show image'),
        required=False)

    acquire_address = schema.Bool(
        title=_(u'label_acquire_address', default=u'Acquire address'),
        required=True)

    dexteritytextindexer.searchable('address')
    address = schema.Text(
        title=_(u'label_address', default=u'Address'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('postal_code')
    postal_code = schema.TextLine(
        title=_(u'label_postal_code', default=u'Postal code'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('city')
    city = schema.TextLine(
        title=_(u'label_city', default=u'City'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('email')
    email = RFC822MailAddress(
        title=_(u'label_email', default=u'E-Mail'),
        missing_value=u"",
        required=False)

    dexteritytextindexer.searchable('phone_office')
    phone_office = schema.TextLine(
        title=_(u'label_phone_office', default=u'Office phone number'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('phone_mobile')
    phone_mobile = schema.TextLine(
        title=_(u'label_phone_mobile', default=u'Mobile phone number'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('fax')
    fax = schema.TextLine(
        title=_(u'label_fax', default=u'Fax number'),
        missing_value=u"",
        default=u"",
        required=False)

    www = schema.URI(
        title=_(u'label_www', default=u'www'),
        missing_value=u"",
        required=False)

    model.fieldset(
        'contact',
        label=_(u'label_fieldset_contact', default=u"Contact"),
        fields=[
            'show_image',
            'acquire_address',
            'address',
            'postal_code',
            'city',
            'phone_office',
            'phone_mobile',
            'fax',
            'email',
            'www']
    )


class MemberBlock(Item):
    """A member
    """
    implements(IMemberBlock)

    def organization(self):
        return get_organization(
            self, ['OrgUnit', 'ftw.simplelayout.ContentPage'])



from AccessControl.SecurityManagement import newSecurityManager
from ftw.contacts.contents.contact import IContactSchema
from ftw.contacts.interfaces import IContact, IContactsSettings
from ftw.contacts.interfaces import ILDAPAttributeMapper
from ftw.contacts.interfaces import ILDAPCustomUpdater
from ftw.contacts.interfaces import ILDAPSearch
from ftw.contacts.sync.mapper import DefaultLDAPAttributeMapper
from plone.app.blob.interfaces import IBlobWrapper
from plone.dexterity.utils import addContentToContainer
from plone.dexterity.utils import iterSchemata
from plone.dexterity.utils import createContent
from plone.namedfile import NamedBlobImage
from plone.namedfile.interfaces import INamedImageField
from plone import api
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import IPloneSiteRoot
from zExceptions import BadRequest
from zope.component import getAdapters
from zope.component import getUtility
from zope.component import queryUtility
from zope.container.interfaces import INameChooser
from zope.event import notify
from zope.lifecycleevent import ObjectAddedEvent
from zope.lifecycleevent import ObjectModifiedEvent
from zope.schema import getFieldsInOrder
from zope.site.hooks import setSite
import argparse
import json
import ldif
import logging
import sys
import transaction


logger = logging.getLogger('ftw.contacts.sync')


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-p', dest='plone_site', default=None,
                        help='Absolute path to the Plone site')
    parser.add_argument('-c', dest='config_file_path', default=None,
                        help='Absolute path to the config file')
    parser.add_argument('-b', dest='base_dn', default=None,
                        help='Base DN for contacts')
    parser.add_argument('-f', dest='filter', default='(objectClass=*)',
                        help='LDAP Filter, defaults to (objectClass=*)')
    parser.add_argument('-l', dest='ldif_file', type=str,
                        help='Import records from the given LDIF file')
    parser.add_argument('-v', dest='verbose', action='store_true',
                        help='Verbose')
    parser.add_argument('-n', dest='dry_run', action='store_true',
                        help='Dry run')
    parser.add_argument('-q', dest='quiet', action='store_true',
                        help='Quiet')

    if sys.argv == ['']:
        options = parser.parse_args([])
    else:
        options = parser.parse_args(sys.argv)

    # Setup logging
    log_handler = logging.StreamHandler()
    log_formatter = logging.Formatter("%(message)s")
    log_handler.setFormatter(log_formatter)
    logger.addHandler(log_handler)
    logger.setLevel(logging.INFO)
    if options.verbose:
        logger.setLevel(logging.DEBUG)

    # If no plone site was provided by the command line, try to find one.
    if options.plone_site is None:
        sites = get_plone_sites(app)
        if len(sites) == 1:
            portal = sites[0]
        elif len(sites) > 1:
            sys.exit("Multiple Plone sites found. Please specify which Plone "
                     "site should be used.")
        else:
            sys.exit("No Plone site found.")
    else:
        portal = app.unrestrictedTraverse(options.plone_site, None)
    if not portal:
        sys.exit("Plone site not found at %s" % options.plone_site)

    user = portal.getOwner()
    newSecurityManager(app, user)

    setSite(portal)

    # get the contact folder
    contacts_path = api.portal.get_registry_record(
        name='contacts_path', interface=IContactsSettings)
    if not contacts_path:
        sys.exit("Contacts path not set. Please configure the path to the "
                 "folder containing contacts in the configuration registry.")
    contacts_folder = portal.unrestrictedTraverse(
        contacts_path.lstrip('/').encode('utf-8'), None)

    if contacts_folder is None:
        sys.exit("Contacts folder not found at %s.")

    # collect config
    config = []
    # read the config from the provided file
    if options.config_file_path:
        with open(options.config_file_path) as config_file:
            config = json.load(config_file)
            # validate
            for entry in config:
                if not entry.get('ldap_plugin_id', False):
                    sys.exit("Please provide the ldap_plugin_id"
                             "in the config file.")

    # or use registry and command line parameter
    else:
        config.append({
            'ldap_plugin_id': api.portal.get_registry_record(
                name='ldap_plugin_id', interface=IContactsSettings),
            'base_dn': options.base_dn,
            'filter': options.filter
        })

    # Read records from an LDIF file
    if options.ldif_file:
        rlist = ldif.LDIFRecordList(open(options.ldif_file, 'rb'))
        rlist.parse()
        ldap_records = rlist.all_records

    # Get records from LDAP
    else:
        ldap = getUtility(ILDAPSearch)
        ldap_records = []

        for ldap_config in config:
            plugin_records = ldap.search(
                plugin_id=ldap_config.get('ldap_plugin_id'),
                base_dn=ldap_config.get('base_dn'),
                filter=ldap_config.get('filter'),
                attrs=get_ldap_attribute_mapper().mapping().keys(),
            )

            # prepend the prefix to avoid id collisions
            if ldap_config.get('userid_prefix'):
                for dn, entry in plugin_records:
                    if not dn:
                        continue
                    id_arr = entry.get(get_ldap_attribute_mapper().id())
                    if id_arr:
                        id_arr[0] = ldap_config.get('userid_prefix').encode('ascii') + id_arr[0]

            ldap_records += plugin_records

    stats = sync_contacts(contacts_folder, ldap_records)

    if not options.dry_run:
        transaction.commit()

    if not options.quiet:
        print "Created: %s" % stats['created']
        print "Modified: %s" % stats['modified']
        print "Unchanged: %s" % stats['unchanged']
        print "Skipped: %s" % stats['skipped']
        print "Failed: %s" % stats['failed']
        print "Total: %s" % stats['total']
        print "Deleted contacts: %s" % stats['deleted']


def get_ldap_attribute_mapper():
    mapper = queryUtility(ILDAPAttributeMapper)
    if mapper is None:
        mapper = DefaultLDAPAttributeMapper()
    return mapper


def get_plone_sites(root):
    result = []
    for obj in root.values():
        if obj.meta_type is 'Folder':
            result = result + get_plone_sites(obj)
        elif IPloneSiteRoot.providedBy(obj):
            result.append(obj)
        elif obj.getId() in getattr(root, '_mount_points', {}):
            result.extend(get_plone_sites(obj))
    return result


def sync_contacts(context, ldap_records, set_owner=False):
    """Synchronize the given ldap results """

    # Statistics
    created = 0
    modified = 0
    skipped = 0
    failed = 0
    deleted = 0

    dn_contact_id_mapping = {}

    ct = getToolByName(context, 'portal_catalog')
    mapper = get_ldap_attribute_mapper()
    dummy_contact = createContent('ftw.contacts.Contact')
    dummy_contact_folder = createContent('ftw.contacts.ContactFolder')

    # 1st pass: create or update profiles
    for dn, entry in ldap_records:

        if not dn:
            continue

        dn = dn.decode('unicode_escape').encode('iso8859-1').decode('utf-8')

        # Only entries with a contact id
        contact_id = entry.get(mapper.id(), [None, ])[0]
        if not contact_id:
            skipped += 1
            logger.debug("Skipping entry '%s'. No contact id." % dn)
            continue
        # Get the normalzied name for the contact with the given id. This has
        # to be done on a dummy folder because existing content would influence
        # the resulting id.
        contact_id = INameChooser(dummy_contact_folder).chooseName(
            contact_id, dummy_contact)

        dn_contact_id_mapping[dn] = contact_id

        contact = context.unrestrictedTraverse(contact_id, None)
        changed = False
        is_new_object = False

        # Check if we really got the wanted object.
        if not IContact.providedBy(contact):
            contact = None

        # Create contact
        if contact is None:
            try:
                contact = createContent('ftw.contacts.Contact')
                contact.id = contact_id
                addContentToContainer(context, contact)

                is_new_object = True
            # invalid id
            except BadRequest:
                failed += 1
                logger.warn("Could not create contact '%s' (invalid id)."
                            % contact_id)
                continue

        # Update/set field values
        IContactSchema(contact).ldap_dn = dn
        field_mapping = {}
        for schemata in iterSchemata(contact):
            for name, field in getFieldsInOrder(schemata):
                field_mapping[name] = field
        for ldap_name, field_name in mapper.mapping().items():
            field = field_mapping.get(field_name, None)
            if field is None:
                raise NotImplementedError()

            value = entry.get(ldap_name, [''])[0]

            current_value = field.get(contact)
            if IBlobWrapper.providedBy(current_value):
                current_value = current_value.data

            if current_value != value:
                # Handle images
                if INamedImageField.providedBy(field) and value:
                    value = NamedBlobImage(
                        data=value,
                        contentType='image/jpeg',
                        filename=u'%s.jpg' % contact_id
                    )
                field.set(contact, value)
                changed = True

        # Update/set fields with custom updaters
        custom_updaters = getAdapters((contact, entry),
                                      provided=ILDAPCustomUpdater)
        for name, updater in custom_updaters:
            changed = updater.update()

        if is_new_object:
            if set_owner:
                # Grant owner role to contact
                contact.__ac_local_roles__ = None
                contact.manage_setLocalRoles(contact_id, ['Owner'])
                contact.reindexObjectSecurity()

            aq_contact = context.get(contact_id)
            ct.catalog_object(aq_contact, '/'.join(aq_contact.getPhysicalPath()))

            notify(ObjectAddedEvent(aq_contact))

            created += 1
            logger.debug("Created new contact '%s (%s)'." % (contact_id, dn))

        elif changed:
            contact.reindexObject()
            notify(ObjectModifiedEvent(contact))
            modified += 1
            logger.debug("Modified contact '%s' (%s)." % (contact_id, dn))

    total = len(ldap_records)
    unchanged = total - skipped - modified - created - failed

    # 2nd pass: set references
    # TODO

    # 3rd pass: delete contacts which have an ldap_id but are not in LDAP.
    all_contacts = ct.unrestrictedSearchResults(
        portal_type='ftw.contacts.Contact',
        path=dict(query='/'.join(context.getPhysicalPath()), depth=1))
    to_be_deleted = {}
    for contact in all_contacts:
        obj = contact.getObject()
        ldap_dn = IContactSchema(obj).ldap_dn
        if ldap_dn and ldap_dn not in dn_contact_id_mapping:
            parent_path = '/'.join(obj.getPhysicalPath()[:-1])
            id_ = obj.getPhysicalPath()[-1]
            if parent_path not in to_be_deleted:
                to_be_deleted[parent_path] = []
            to_be_deleted[parent_path].append(id_)
            logger.debug("Deleting contact '%s'" % id_)

    # Disable link integrity check while deleting contacts
    ptool = getToolByName(context, 'portal_properties')
    props = getattr(ptool, 'site_properties')
    old_check = props.getProperty('enable_link_integrity_checks', False)
    props.enable_link_integrity_checks = False

    for parent_path, ids in to_be_deleted.items():
        parent = context.unrestrictedTraverse(parent_path)
        deleted += len(ids)
        parent.manage_delObjects(ids)

    # Reenable previous link integrity setting
    props.enable_link_integrity_checks = old_check

    return dict(
        created=created,
        modified=modified,
        unchanged=unchanged,
        total=total,
        skipped=skipped,
        failed=failed,
        deleted=deleted,
    )


if __name__ == '__main__':
    main()



import sys
import os


def do_sync_profiles(self, args):
    script = "{0}/sync.py".format(os.path.split(__file__)[0])
    # execfile() needs the source file
    if script.endswith('.pyc') or script.endswith('.pyo'):
        script = script[:-1]
    cmd = "import sys; sys.argv[:]=[];"
    if len(self.options.args) > 1:
        for arg in self.options.args[1:]:
            cmd += 'sys.argv.append(r\'%s\');' % arg
    cmd += 'execfile(r\'%s\')' % script
    cmdline = self.get_startup_cmd(self.options.python, cmd)
    exitstatus = os.system(cmdline)
    sys.exit(exitstatus)



from ftw.builder import Builder
from ftw.builder import create
from ftw.contacts.contents.contact import IContactSchema
from ftw.contacts.interfaces import IContact
from ftw.contacts.interfaces import ILDAPAttributeMapper
from ftw.contacts.interfaces import ILDAPCustomUpdater
from ftw.contacts.sync.sync import get_ldap_attribute_mapper
from ftw.contacts.sync.sync import sync_contacts
from ftw.contacts.tests import FunctionalTestCase
from zope.component import adapts
from zope.component import getGlobalSiteManager
from zope.component import provideAdapter
from zope.component import provideUtility
from zope.interface import implements
from zope.interface import Interface
import ldif
import os.path


class TestContactSynchronization(FunctionalTestCase):

    def setUp(self):
        super(TestContactSynchronization, self).setUp()
        self.grant('Manager')
        self.contacts = create(Builder('contact folder').titled(u'Contacts'))

    def test_sync_creates_contacts(self):
        res = sync_contacts(self.contacts, get_ldif_records('contacts.ldif'))
        self.assertEquals(5, res['created'])
        self.assertEquals(5, len(self.contacts.objectIds()))

    def test_sync_creates_contact_with_attributes(self):
        res = sync_contacts(self.contacts, get_ldif_records('contact.ldif'))
        self.assertEquals(1, res['created'])

        contact = self.contacts['nina-mueller']
        self.assertEquals(u'cn=M\xfcller Nina,ou=Payroll,dc=domain, dc=net',
                          IContactSchema(contact).ldap_dn)
        self.assertEquals('Nina', IContactSchema(contact).firstname)
        self.assertEquals('M\xc3\xbcller', IContactSchema(contact).lastname)
        self.assertEquals('4teamwork AG', IContactSchema(contact).organization)
        self.assertEquals('n.mueller@4teamwork.ch', IContactSchema(contact).email)
        self.assertEquals('Biel', IContactSchema(contact).city)

    def test_sync_updates_contact(self):
        contact = create(Builder('contact').within(self.contacts).having(
            _id='nina.mueller',
            ldap_dn=u'cn=M\u00fcller Nina,ou=Payroll,dc=domain, dc=net',
            firstname='Nina',
            lastname='Meier',
            ))
        self.contacts.manage_renameObject(contact.id, 'nina-mueller')
        res = sync_contacts(self.contacts, get_ldif_records('contact.ldif'))
        self.assertEquals(1, res['modified'])
        self.assertEquals('Nina', IContactSchema(contact).firstname)
        self.assertEquals('M\xc3\xbcller', IContactSchema(contact).lastname)
        self.assertEquals('4teamwork AG', IContactSchema(contact).organization)
        self.assertEquals('n.mueller@4teamwork.ch', IContactSchema(contact).email)

    def test_sync_contact_without_changes(self):
        res = sync_contacts(self.contacts, get_ldif_records('contact.ldif'))
        res = sync_contacts(self.contacts, get_ldif_records('contact.ldif'))
        self.assertEquals(1, res['unchanged'])

    def test_sync_deletes_contact(self):
        create(Builder('contact').within(self.contacts).having(
            _id='julia.meier',
            ldap_dn=u'cn=Meier Julia,ou=Payroll,dc=domain, dc=net',
        ))
        res = sync_contacts(self.contacts, get_ldif_records('contact.ldif'))
        self.assertEquals(1, res['deleted'])
        self.assertNotIn('julia.meier', self.contacts.objectIds())

    def test_sync_reindexes_updated_contact(self):
        contact = create(Builder('contact').within(self.contacts).having(
            _id='nina-mueller',
            ldap_dn=u'cn=M\u00fcller Nina,ou=Payroll,dc=domain, dc=net',
            firstname='Nina',
            lastname='Meier',
            ))
        self.contacts.manage_renameObject(contact.id, 'nina-mueller')
        res = sync_contacts(self.contacts, get_ldif_records('contact.ldif'))
        self.assertEquals(1, res['modified'])
        catalog_results = self.portal.portal_catalog(Title='M\xc3\xbcller')
        self.assertEquals(1, len(catalog_results))
        self.assertEquals('M\xc3\xbcller Nina', catalog_results[0].Title)


class TestLDAPAttributeMapper(FunctionalTestCase):

    def test_default_mapper(self):
        mapper = get_ldap_attribute_mapper()
        self.assertEquals('uid', mapper.id())
        self.assertEquals('lastname', mapper.mapping()['sn'])

    def test_custom_mapper(self):
        class MyLDAPAttributeMapper(object):
            implements(ILDAPAttributeMapper)

            def mapping(self):
                return {'sAMAccountName': 'userid'}

            def id(self):
                return 'sAMAccountName'

        my_mapper = MyLDAPAttributeMapper()
        provideUtility(my_mapper)
        mapper = get_ldap_attribute_mapper()

        self.assertEquals('sAMAccountName', mapper.id())
        self.assertEquals('userid', mapper.mapping()['sAMAccountName'])

        # cleanup
        getGlobalSiteManager().unregisterUtility(my_mapper, ILDAPAttributeMapper)


class TestCustomUpdater(FunctionalTestCase):

    class MyCustomUpdater(object):
        implements(ILDAPCustomUpdater)
        adapts(IContact, Interface)

        def __init__(self, contact, record):
            self.contact = contact
            self.record = record

        def update(self):
            IContactSchema(self.contact).firstname = 'Tania'
            return True

    def setUp(self):
        super(TestCustomUpdater, self).setUp()
        self.grant('Manager')
        self.contacts = create(Builder('contact folder').titled(u'Contacts'))
        provideAdapter(TestCustomUpdater.MyCustomUpdater)

    def tearDown(self):
        getGlobalSiteManager().unregisterAdapter(TestCustomUpdater.MyCustomUpdater)

    def test_custom_updater_updates_field(self):
        sync_contacts(self.contacts, get_ldif_records('contact.ldif'))
        self.assertEquals(
            'Tania',
            IContactSchema(self.contacts['nina-mueller']).firstname)


def get_ldif_records(filename):
    path = os.path.join(os.path.dirname(__file__), 'assets', filename)
    rlist = ldif.LDIFRecordList(open(path, 'rb'))
    rlist.parse()
    return rlist.all_records



from setuptools import find_packages
from setuptools import setup

import os

version = '1.8.0'
maintainer = 'Mathias Leimgruber'

tests_require = [
    'ftw.builder',
    'plone.app.testing',
    'ftw.testbrowser',
    'ftw.contacts [zip_export, simplelayout, geo, ldap]',
    'unittest2',
    ]
zip_export = [
    'ftw.zipexport',
    ]
simplelayout = [
    'ftw.simplelayout [contenttypes]',
    'ftw.referencewidget',
    ]
geo = [
    'ftw.geo',
    'collective.geo.openlayers >= 3.2b1',
    'collective.geo.bundle [dexterity]',
    ]
ldap = [
    'python-ldap<=2.4.25',
    'Products.PloneLDAP'
    ]
extras_require = {
    'tests': tests_require,
    'zip_export': zip_export,
    'simplelayout': simplelayout,
    'geo': geo,
    'ldap': ldap,
    }

setup(name='ftw.contacts',
      version=version,
      description='Provides a contact-contenttype',
      long_description=open(
          'README.rst').read() + '\n' + open(
          os.path.join('docs', 'HISTORY.txt')).read(),

      # Get more strings from
      # http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          'Framework :: Plone',
          'Framework :: Plone :: 4.2',
          'License :: OSI Approved :: GNU General Public License (GPL)',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'Topic :: Software Development :: Libraries :: Python Modules',
          ],

      keywords='ftw plone contacts',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      maintainer=maintainer,
      url='https://github.com/4teamwork/ftw.contacts',
      license='GPL2',

      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw', ],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
          'Plone',
          'z3c.schema',
          'setuptools',
          'collective.dexteritytextindexer',
          'collective.js.jqueryui',
          'plone.app.relationfield',
          'plone.app.referenceablebehavior',
          'plone.api >= 1.3.3',
          'ftw.zipexport',
          'ftw.autofeature',
          'ftw.upgrade',
          ],

      tests_require=tests_require,
      extras_require=extras_require,

      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone

      [plone.recipe.zope2instance.ctl]
      sync_contacts = ftw.contacts.sync.command:do_sync_profiles
      """,
      )



from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from ftw.contacts.tests import builders
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2
from zope.configuration import xmlconfig


class ContactsLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '</configure>',
            context=configurationContext)

        z2.installProduct(app, 'ftw.simplelayout')

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'ftw.simplelayout.contenttypes:default')
        applyProfile(portal, 'ftw.contacts.simplelayout:default')
        applyProfile(portal, 'ftw.zipexport:default')


FTW_CONTACTS_FIXTURE = ContactsLayer()
FTW_CONTACTS_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FTW_CONTACTS_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.contacts:functional")


class ContactsGeoLayer(ContactsLayer):

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'ftw.contacts.geo:default')


FTW_CONTACTS_GEO_FIXTURE = ContactsGeoLayer()
FTW_CONTACTS_GEO_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FTW_CONTACTS_GEO_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.contacts.geo:functional")



from ftw.contacts.testing import FTW_CONTACTS_FUNCTIONAL_TESTING
from ftw.contacts.utils import normalized_first_letter
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest2 import TestCase


class TestNormalizedFirstLetter(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_returns_uppercase_letter(self):
        self.assertEquals('T', normalized_first_letter('test'))

    def test_normalizes_umlauts(self):
        self.assertEquals('A', normalized_first_letter('\xc3\xa4bc'))



from collective import dexteritytextindexer
from ftw.contacts import _
from ftw.contacts.interfaces import IContact
from ftw.contacts.utils import get_contact_title
from plone.app.textfield import RichText
from plone.autoform import directives
from plone.dexterity.content import Item
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import model
from z3c.schema.email import RFC822MailAddress
from zope import schema
from zope.interface import implements
from zope.interface import Invalid
from zope.interface import invariant
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class NoOrganizationOrFullname(Invalid):
    """The organization or fullname is invalid
    """


gender_choice_vocabulary = SimpleVocabulary([
    SimpleTerm(value=u'm', title=_(u'label_male', default=u'Male')),
    SimpleTerm(value=u'f', title=_(u'label_female', default=u'Female')),
    SimpleTerm(value=u'', title=_(u'label_gender_neutral', default=u'-'))
    ])


class IContactSchema(model.Schema):
    """A contact type schema interface
    """
    dexteritytextindexer.searchable('organization')
    organization = schema.TextLine(
        title=_(u'label_organization', default=u'Organization'),
        missing_value=u"",
        default=u"",
        required=False)

    gender = schema.Choice(
        title=_(u'label_gender', default=u'Gender'),
        vocabulary=gender_choice_vocabulary,
        required=True,
        default="")

    dexteritytextindexer.searchable('lastname')
    lastname = schema.TextLine(
        title=_(u'label_lastname', default=u'Lastname'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('firstname')
    firstname = schema.TextLine(
        title=_(u'label_firstname', default=u'Firstname'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('address')
    address = schema.Text(
        title=_(u'label_address', default=u'Address'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('postal_code')
    postal_code = schema.TextLine(
        title=_(u'label_postal_code', default=u'Postal code'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('city')
    city = schema.TextLine(
        title=_(u'label_city', default=u'City'),
        missing_value=u"",
        default=u"",
        required=False)

    directives.mode(country='hidden')
    country = schema.TextLine(
        title=_(u'label_country', default=u'Country'),
        required=False,
        default=u"Schweiz")

    dexteritytextindexer.searchable('email')
    email = RFC822MailAddress(
        title=_(u'label_email', default=u'E-Mail'),
        missing_value=u"",
        required=False)

    dexteritytextindexer.searchable('phone_office')
    phone_office = schema.TextLine(
        title=_(u'label_phone_office', default=u'Office phone number'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('phone_mobile')
    phone_mobile = schema.TextLine(
        title=_(u'label_phone_mobile', default=u'Mobile phone number'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('fax')
    fax = schema.TextLine(
        title=_(u'label_fax', default=u'Fax number'),
        missing_value=u"",
        default=u"",
        required=False)

    www = schema.URI(
        title=_(u'label_www', default=u'www'),
        missing_value=u"",
        required=False)

    image = NamedBlobImage(
        title=_(u'label_image', default=u'Image'),
        missing_value=None,
        required=False)

    academic_title = schema.TextLine(
        title=_(u'label_academic_title', default=u'Academic title'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('function')
    function = schema.TextLine(
        title=_(u'label_function', default=u'Function'),
        description=_(
            u'description_function',
            default=u'Will be displayed even if the address is '
                    u'acquired and the contact has a function.'
        ),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('department')
    department = schema.TextLine(
        title=_(u'label_department', default=u'Department'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('salutation')
    salutation = schema.TextLine(
        title=_(u'label_salutation', default=u'Salutation'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('text')
    text = RichText(
        title=_(u'label_text', default=u'Text'),
        required=False,
        allowed_mime_types=('text/html',))

    dexteritytextindexer.searchable('phone_private')
    phone_private = schema.TextLine(
        title=_(u'label_phone_private', default=u'Private phone number'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('address_private')
    address_private = schema.Text(
        title=_(u'label_address_private', default=u'Address'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('postal_code_private')
    postal_code_private = schema.TextLine(
        title=_(u'label_postal_code_private', default=u'Postal code'),
        missing_value=u"",
        default=u"",
        required=False)

    dexteritytextindexer.searchable('city_private')
    city_private = schema.TextLine(
        title=_(u'label_city_private', default=u'City'),
        missing_value=u"",
        default=u"",
        required=False)

    directives.mode(uid='hidden')
    uid = schema.TextLine(
        title=_(u'label_uid', default=u'UID'),
        missing_value=u"",
        default=u"",
        required=False)

    directives.mode(ldap_dn='hidden')
    ldap_dn = schema.TextLine(
        title=_(u'label_ldap_dn', default=u'LDAP DN'),
        missing_value=u"",
        default=u"",
        required=False)

    hide_memberships = schema.Bool(
        title=_(u'label_hide_memberships', default=u'Hide memberships'),
        default=False,
        required=False)

    hide_map = schema.Bool(
        title=_(u'label_hide_map', default=u'Hide map'),
        default=False,
        required=False)

    model.fieldset(
        'extended',
        label=_(u'label_fieldset_extended', default=u"Extended"),
        fields=[
            'image',
            'academic_title',
            'function',
            'department',
            'salutation',
            'text',
            'hide_memberships',
            'hide_map',
        ]
    )

    model.fieldset(
        'private',
        label=_(u'label_fieldset_homeaddress', default=u"Home address"),
        fields=[
            'phone_private',
            'address_private',
            'postal_code_private',
            'city_private']
    )

    @invariant
    def validateOrganizationOrFullname(data):
        if data.organization or (data.firstname and data.lastname):
            return None

        raise NoOrganizationOrFullname(_(
            u'validation_error_organization_or_name_required',
            default=u'Either the name of the organization or first- '
            u'and lastname of the person is required.')
        )


class Contact(Item):
    """A contact
    """
    implements(IContact)

    @property
    def title(self):
        """Getter function for the title

        The contact title is generated from the firstname and lastname
        or from the organization name. Depends on what fields are filled.
        """
        return get_contact_title(self)

    def setTitle(self, value):
        return



from ftw.contacts.interfaces import IContact
from ftw.contacts.utils import ALPHABET
from ftw.contacts.utils import AlphabeticLetters
from ftw.contacts.utils import normalized_first_letter
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView

import json
import re


class ContactFolderView(BrowserView):
    """Contactfolder view
    """


class Letters(BrowserView):
    """View to render the letters-listing
    """
    def __call__(self, brains, current_letter):
        self.letters = AlphabeticLetters().letters(current_letter, brains)
        return super(Letters, self).__call__()


class ContactFolderReload(BrowserView):
    """Returns the nessesary html elements to reload the
    contactlisting per ajax.

    Checks the request for:

    - searchable_text
    - letter
    - index_from
    - index_to

    Contacts:
    This view will be called asynchron to lazy-load the contacts.
    It returns the next few full rendered contacts
    for the contactfolderlisting.

    Letters:
    Returns the letterlisting.
    Puts css-classes on each letter which differs between the
    current letter and letters with contacts.
    """
    def __call__(self):
        self.request.response.setHeader('X-Theme-Disabled', 'True')
        self.request.response.setHeader(
            'Content-Type', 'application/json; charset=utf-8')
        index_from = int(self.request.form.get('index_from', 0))
        index_to = int(self.request.form.get('index_to', 50))
        searchable_text = self.request.form.get('searchable_text', '')
        current_letter = self.request.form.get('letter', None)

        brains = self._contact_brains(SearchableText=searchable_text)
        filtered_brains = self._filter_letter(brains, current_letter)

        # max_contacts: is the length of brains after filtered by
        #               the searchable text and filtered by the letters
        # contacts: are the rendered contacts, sliced with the
        #           index_from and index_to parameter
        # letters: are the rendered letters only filterd by the
        #          searchable_text to get all the possible letters
        return json.dumps({
            'max_contacts': len(filtered_brains),
            'contacts': self.rendered_contacts(
                filtered_brains, index_from, index_to),
            'letters': self.rendered_letters(brains, current_letter)})

    def rendered_contacts(
            self, filtered_brains, index_from, index_to):
        brains = filtered_brains[index_from:index_to]

        return ''.join([brain.getObject().restrictedTraverse(
            '@@contact_summary')() for brain in brains])

    def rendered_letters(self, brains, current_letter):
        view = self.context.restrictedTraverse('@@letters')
        return view(brains, current_letter)

    def _contact_brains(self, **query):
        query = self._build_query(**query)
        self.catalog = getToolByName(self.context, 'portal_catalog')

        return self.catalog(query)

    def _filter_letter(self, brains, current_letter):
        """Compares the title of the brain with the letter and
        filter matching elements
        """
        if not current_letter:
            return brains

        filtered_brains = []
        for brain in brains:
            letter = normalized_first_letter(brain.Title)
            if letter == current_letter:
                filtered_brains.append(brain)
            elif current_letter == '#' and letter not in ALPHABET:
                filtered_brains.append(brain)

        return filtered_brains

    def _build_query(self, **query):
        folder_path = '/'.join(self.context.getPhysicalPath())
        basequery = dict(
            path={'query': folder_path},
            sort_on='sortable_title',
            object_provides=IContact.__identifier__,
            SearchableText='')

        basequery.update(query)
        return self._cleanup_query(basequery)

    def _cleanup_query(self, query):
        text = query.get('SearchableText')

        # Remove unsupported words
        text = re.sub(re.compile(
            r'[^\w\s]', re.UNICODE), r'', text.decode('utf-8')).encode('utf-8')

        text = text.strip()

        if not text:
            query.pop('SearchableText')
            return query

        # Add wildcard
        text = "{0}*".format(text)

        query.update({'SearchableText': text})

        return query



from ftw.builder import Builder
from ftw.builder import create
from ftw.contacts.browser.contactfolder import ContactFolderReload
from ftw.contacts.testing import FTW_CONTACTS_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from Products.CMFCore.utils import getToolByName
from unittest2 import TestCase
import json


class TestContactFolderReloadLetter(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.catalog = getToolByName(self.portal, 'portal_catalog')

        self.contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))

        self.view = self.contactfolder.unrestrictedTraverse('reload_contacts')

    @browsing
    def test_returns_html_letters(self, browser):
        create(Builder('contact')
               .with_minimal_info(u'Chuck', u'4orris')
               .within(self.contactfolder))

        create(Builder('contact')
               .with_minimal_info(u'J\xe4mes', u'B\xf6nd')
               .within(self.contactfolder))

        brains = self.catalog(portal_type="ftw.contacts.Contact")

        browser.open_html(self.view.rendered_letters(brains, 'B'))

        self.assertEqual(2, len(browser.css('.letter.withContent')))
        self.assertEqual(1, len(browser.css('.letter.active')))


class TestContactFolderReloadContact(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.catalog = getToolByName(self.portal, 'portal_catalog')

        self.contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))

        self.view = self.contactfolder.unrestrictedTraverse('reload_contacts')

    @browsing
    def test_returns_html_contacts(self, browser):
        create(Builder('contact')
               .with_minimal_info(u'Chuck', u'4orris')
               .within(self.contactfolder))

        create(Builder('contact')
               .with_minimal_info(u'J\xe4mes', u'B\xf6nd')
               .within(self.contactfolder))

        brains = self.catalog(portal_type="ftw.contacts.Contact")

        browser.open_html(self.view.rendered_contacts(
            brains, 0, 10))

        self.assertEqual(2, len(browser.css('.contactSummary')))

    @browsing
    def test_restrict_from_and_to(self, browser):
        create(Builder('contact')
               .with_minimal_info(u'Chuck', u'4orris')
               .within(self.contactfolder))

        create(Builder('contact')
               .with_minimal_info(u'J\xe4mes', u'B\xf6nd')
               .within(self.contactfolder))

        create(Builder('contact')
               .with_minimal_info(u'Max', u'Muster')
               .within(self.contactfolder))

        brains = self.catalog(portal_type="ftw.contacts.Contact")

        browser.open_html(self.view.rendered_contacts(brains, 1, 3))

        self.assertEqual(
            2, len(browser.css('.contactSummary')),
            "Should return only the second and third contact")

        browser.open_html(self.view.rendered_contacts(brains, 0, 1))

        self.assertEqual(
            1, len(browser.css('.contactSummary')),
            "Should return only the first contact")


class TestContactFolderReloadView(TestCase):

    layer = FTW_CONTACTS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.catalog = getToolByName(self.portal, 'portal_catalog')

        self.contactfolder = create(
            Builder('contact folder').titled(u'Contact folder'))

        self.view = self.contactfolder.unrestrictedTraverse('reload_contacts')

    @browsing
    def test_json_string_on_call(self, browser):
        create(Builder('contact')
               .with_minimal_info(u'Chuck', u'4orris')
               .within(self.contactfolder))

        create(Builder('contact')
               .with_minimal_info(u'J\xe4mes', u'B\xf6nd')
               .within(self.contactfolder))

        create(Builder('contact')
               .with_minimal_info(u'Max', u'Muster')
               .within(self.contactfolder))

        data = json.loads(self.view())

        self.assertEqual(3, data.get('max_contacts'))

        browser.open_html(data.get('contacts'))

        self.assertEqual(3, len(browser.css('.contactSummary')))

        browser.open_html(data.get('letters'))

        self.assertEqual(3, len(browser.css('.letter.withContent')))

    @browsing
    def test_restrict_contacts_with_searchable_text(self, browser):
        create(Builder('contact')
               .with_minimal_info(u'Chuck', u'4orris')
               .within(self.contactfolder))

        create(Builder('contact')
               .with_minimal_info(u'J\xe4mes', u'B\xf6nd')
               .within(self.contactfolder))

        create(Builder('contact')
               .with_minimal_info(u'Max', u'Muster')
               .within(self.contactfolder))

        self.request.form['searchable_text'] = "B\xc3\xb6"

        view = ContactFolderReload(self.contactfolder, self.request)
        data = json.loads(view())

        self.assertEqual(1, data.get('max_contacts'))

        browser.open_html(data.get('contacts'))

        self.assertEqual(1, len(browser.css('.contactSummary')))

        browser.open_html(data.get('letters'))

        self.assertEqual(1, len(browser.css('.letter.withContent')))

    @browsing
    def test_search_with_special_chars(self, browser):
        create(Builder('contact')
               .with_minimal_info(u'Chuck', u'4orris')
               .within(self.contactfolder))

        self.request.form['searchable_text'] = "Ch/*+uck 4or"

        view = ContactFolderReload(self.contactfolder, self.request)
        data = json.loads(view())

        self.assertEqual(
            1, data.get('max_contacts'),
            "There should be one contact, the 'Chuck 4orris'. " +
            "The special chars are replaced with an empty string.")

    @browsing
    def test_search_only_special_chars(self, browser):
        create(Builder('contact')
               .with_minimal_info(u'Chuck', u'4orris')
               .within(self.contactfolder))

        self.request.form['searchable_text'] = "  /*+  "

        view = ContactFolderReload(self.contactfolder, self.request)
        data = json.loads(view())

        self.assertEqual(
            1, data.get('max_contacts'),
            "There should be all contacts." +
            "The special chars are replaced with an empty string.")

    @browsing
    def test_search_with_empty_string(self, browser):
        create(Builder('contact')
               .with_minimal_info(u'Chuck', u'4orris')
               .within(self.contactfolder))

        self.request.form['searchable_text'] = ""

        view = ContactFolderReload(self.contactfolder, self.request)
        data = json.loads(view())

        self.assertEqual(
            1, data.get('max_contacts'), "There should be all contacts")

    @browsing
    def test_max_contacts_with_letters_filter(self, browser):
        create(Builder('contact')
               .with_minimal_info(u'Chuck', u'Borris')
               .within(self.contactfolder))

        create(Builder('contact')
               .with_minimal_info(u'J\xe4mes', u'B\xf6nd')
               .within(self.contactfolder))

        create(Builder('contact')
               .with_minimal_info(u'Max', u'Muster')
               .within(self.contactfolder))

        create(Builder('contact')
               .with_minimal_info(u'Max', u'4uster')
               .within(self.contactfolder))

        self.request.form['letter'] = "B"

        view = ContactFolderReload(self.contactfolder, self.request)
        data = json.loads(view())

        self.assertEqual(
            2, data.get('max_contacts'),
            """Becauswe the letter B is active, the max_contacts should
            be only contain the Chuck and James contact, means '2'"""
            )

        browser.open_html(data.get('contacts'))

        self.assertEqual(
            2, len(browser.css('.contactSummary')),
            """Should return the Chuck and james contact because the
            letter filter""")

        browser.open_html(data.get('letters'))

        self.assertEqual(
            3, len(browser.css('.letter.withContent')),
            """Should show all available letters, means the #, M and B""")

    @browsing
    def test_max_contacts_with_special_letters_filter(self, browser):
        create(Builder('contact')
               .with_minimal_info(u'Chuck', u'Borris')
               .within(self.contactfolder))

        create(Builder('contact')
               .with_minimal_info(u'J\xe4mes', u'B\xf6nd')
               .within(self.contactfolder))

        create(Builder('contact')
               .with_minimal_info(u'Bud', u'Muster')
               .within(self.contactfolder))

        create(Builder('contact')
               .with_minimal_info(u'Max', u'4uster')
               .within(self.contactfolder))

        self.request.form['letter'] = "#"

        view = ContactFolderReload(self.contactfolder, self.request)
        data = json.loads(view())

        self.assertEqual(
            1, data.get('max_contacts'),
            """Becauswe the letter # is active, only the contacts
            with a special char should be shown. Means, the Max 4uster"""
            )

        browser.open_html(data.get('contacts'))

        self.assertEqual(
            1, len(browser.css('.contactSummary')),
            """Should return the Max contact because the letter filter""")

        browser.open_html(data.get('letters'))

        self.assertEqual(
            3, len(browser.css('.letter.withContent')),
            """Should show all available letters, means the #, M and B""")



from ftw.contacts.interfaces import IContact
from ftw.contacts.utils import generate_vcard
from ftw.zipexport.interfaces import IZipRepresentation
from ftw.zipexport.representations.general import NullZipRepresentation
from Products.CMFPlone.utils import safe_unicode
from zope.component import adapts
from zope.interface import implements
from zope.interface import Interface


class ContactZipRepresentation(NullZipRepresentation):
    implements(IZipRepresentation)
    adapts(IContact, Interface)

    def get_files(self, path_prefix=u"", recursive=True, toplevel=True):
        filename = safe_unicode('{0}.vcf'.format(self.context.getId()))

        yield (u'{0}/{1}'.format(safe_unicode(path_prefix), filename),
               generate_vcard(self.context))
